/**
 * Copyright (c) 2007, Opera Software ASA
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Opera Software ASA nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY OPERA SOFTWARE ASA AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL OPERA SOFTWARE ASA AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

onload = function() {
  var wikipedia = mediawikiApi();
  
  var KEYS = {
    SPECIAL: set(9, 13, 16, 17, 18, 20, 27, 33, 34, 35, 36, 37, 38, 39, 40, 45,
      112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 144),
    ENTER: 13
  };
  
  function normalizeRomaji(str) {
    return str.replace( /[\u3000\uFF01-\uFF5E]/g, function(m0) {
      var cc = m0.charCodeAt(0);
      return cc == 0x3000 ? ' ' : String.fromCharCode( cc - 0xFEE0 );
    } );
  }
  
  function normalizeKana(str) {
    return str.replace( /([\uFF66-\uFF9D])(\uFF9E?)(\uFF9F?)/g, function(m0, kana, tenten, maru) {
      var off = ( tenten ? 1 : 0 ) + ( maru ? 2 : 0 );
      var cc = m0.charCodeAt(0);
       return String.fromCharCode( kanamap[off][ cc - 0xFF00 ] );
    } );
  }
  
  var kanamap = [
    // normal cases (incomplete)
    { 0x8A:0x30CF, 0x8B:0x30D2, 0x8C:0x30D5, 0x8D:0x30D8, 0x8E:0x30DB },
    // tenten (ﾞ) cases (incomplete)
    { 0x8A:0x30D0, 0x8B:0x30D3, 0x8C:0x30D6, 0x8D:0x30D9, 0x8E:0x30DC },
    // maru (ﾟ) cases (complete)
    { 0x8A:0x30D1, 0x8B:0x30D4, 0x8C:0x30D7, 0x8D:0x30DA, 0x8E:0x30DD },
  ];
  
  /* Some initial setup */
  
  // Translation settings
  // These are the language codes that are used at Wikipedia
  window.languages = ['en', 'ja', 'no', 'sv'];
  
  // This does not take in account the current page, so a history length of '2'
  // means that the back button can be pressed twice
  var historyLength = 15;
  // If true, caches content for faster history browsing. Should be false for
  // devices with limited memory
  var contentCache = true;
  var minimized = true;
  // Error message when the widget get a status of 0
  var errorMessage = '<p><strong>Could not contact Wikipedia</strong></p>' +
    '<p>Please check your Internet connection and that Opera is not in offline mode.</p>';
  // If auto search (Google Suggest-style) should be used
  var autoSearch = true;
  // If auto search is currently activated in the preferences
  var autoSearchActive = true;
  var searchTimeout = null;
  // Animation settings
  var animInterval = null;
  var animationSteps = Math.ceil(screen.availWidth / Math.ceil(screen.availWidth / 80));
  var media = 'screen';
  
  var BACK_BUTTON = 90; // Z
  var FORWARD_BUTTON = 88; // X
  
  // Widget IDs
  var body = document.body;
  var wdgtMain = document.getElementById('wdgt-main');
  var wdgtSearch = document.getElementById('wdgt-search');
  var wdgtQuery = document.getElementById('wdgt-query');
  var wdgtQueryList = document.getElementById('wdgt-query-list');
  var wdgtSearchGo = document.getElementById('wdgt-search-go');
  var wdgtSearchLoader = document.getElementById('wdgt-search-loader');
  var wdgtContent = document.getElementById('wdgt-content');
  var wdgtConfig = document.getElementById('wdgt-config');
  var wdgtToc = document.getElementById('wdgt-toc');
  var wdgtTocContainer = document.getElementById('wdgt-toc-container');
  var wdgtNotoc = document.getElementById('wdgt-notoc');
  var wdgtTemp = document.getElementById('wdgt-temp');
  var wdgtMinmaxButton = document.getElementById('wdgt-minmax-button');
  var wdgtCloseButton = document.getElementById('wdgt-close-button');
  var wdgtConfigButton = document.getElementById('wdgt-config-button');
  var wdgtEditButton = document.getElementById('wdgt-edit-button');
  var wdgtBackButton = document.getElementById('wdgt-back-button');
  var wdgtForwardButton = document.getElementById('wdgt-forward-button');
  var wdgtStatus = document.getElementById('wdgt-status');
  var wdgtTextupButton = document.getElementById('wdgt-textup-button');
  var wdgtTextdownButton = document.getElementById('wdgt-textdown-button');
  var wdgtLangPicker = document.getElementById('wdgt-lang-picker');
  var wdgtLoadMain = document.getElementById('wdgt-load-main');
  var wdgtAutoSearch = document.getElementById('wdgt-auto-search');
  var wdgtUseAutoSearch = document.getElementById('wdgt-use-auto-search');
  var wdgtLoadImages = document.getElementById('wdgt-load-images');
  var wdgtLoadIndicator = document.getElementById('wdgt-load-indicator');
  var wdgtSwitchContent = document.getElementById('wdgt-switch-content');
  var wdgtSwitchToc = document.getElementById('wdgt-switch-toc');
  var wdgtPanelSwitchers = document.getElementById('wdgt-panel-switchers');
  
  
  // Create a temp element where we can dump stuff
  var temp = document.createElement('div');
  wdgtTemp.appendChild(temp);
  
  
  /* Device specific settings */
  
  // enable device style sheet
  if (testMedia().type == 'screen')
    enableStyleSheet('screen');
  else if (testMedia().type == 'handheld')
  {
    enableStyleSheet('handheld');
//    window.resizeTo(screen.availWidth, screen.availHeight);
  }
  else if (testMedia().type == 'tv')
    enableStyleSheet('tv');
    
//  if (mediaQuery('screen')) {
  if (testMedia().type == 'screen') {
    // Don't start minimized on devices with small screen
//    if (mediaQuery('screen and (max-device-width: 800px)')) {
    if (testMedia().type == 'handheld') {
      body.dropClass('minimized');
      minimized = false;
    }
    else {
      window.resizeTo(400, 70);
    }
    wdgtQuery.focus();
    makeWindowResizable();
  }
  
//  if (mediaQuery('handheld, tv')) {
  if (testMedia().type == 'handheld' || testMedia().type == 'tv') {
    // History settings
    historyLength = 5;
    contentCache = false;
    
    // Disable auto search
    autoSearch = false;
    // Fill up the whole screen on TV and handheld devices
    window.resizeTo(screen.availWidth, screen.availHeight);
  }
  
//  if (mediaQuery('handheld')) {
  if (testMedia().type == 'handheld') {
    var BACK_BUTTON = 90; // Z
    var FORWARD_BUTTON = 88; // X
    var media = 'handheld';
    var topBar = 34;
    wdgtMain.style.left = 0;
    wdgtContent.style.width =
      wdgtContent.style.marginLeft =
      wdgtConfig.style.marginLeft =
      screen.availWidth + 'px';
    wdgtToc.style.width = screen.availWidth - 10 + 'px';
    wdgtTocContainer.style.height =
      wdgtContent.style.height =
      wdgtPanelSwitchers.style.height =
      screen.availHeight - topBar + 'px';
  }
  
//  if (mediaQuery('tv')) {
  if (testMedia().type == 'tv') {
    var media = 'tv';
    wdgtBackButton.setAttribute('disabled', 'disabled');
    wdgtForwardButton.setAttribute('disabled', 'disabled');
    
    wdgtToc.style.height =
      wdgtContent.style.height =
      wdgtSwitchContent.height =
      screen.availHeight - 54 + 'px';
  }
  

  
  
  /* Preference handling */
  
  var lang = widget.preferenceForKey('lang') || 'en';
  if (lang != 'en') {
    document.toLocale(lang);
    wikipedia.setLanguage(lang);
  }

  wdgtContent.style.fontSize = widget.preferenceForKey('font-size') || '12px';
  
  var loadMain = widget.preferenceForKey('load-main');
  wdgtLoadMain.checked = loadMain === '' || loadMain == 'false' ? false : true;
  
  var useAutoSearch = widget.preferenceForKey('use-auto-search');
  wdgtUseAutoSearch.checked = useAutoSearch === '' || useAutoSearch == 'true' ? true : false;
  
  var loadImages = widget.preferenceForKey('load-images');
  wdgtLoadImages.checked = loadImages === '' || loadImages == 'true' ? true : false;
  
  // Set the current language in the language picker
  for (var i = 0, option; option = wdgtLangPicker.getElementsByTagName('option')[i]; i++) {
    if (option.value == lang) {
      option.selected = true;
      break;
    }
  }
  
  // Load main page if that preference is set
  if (wdgtLoadMain.checked) {
    body.addClass('loading');
    wikipedia.specialSearch(
      'Main_Page',
      function(request) { prepareArticle(request.responseText); }
    );
  }
  
  if (!wdgtUseAutoSearch.checked) {
    autoSearchActive = false;
  }
  
  if (!wdgtLoadImages.checked) {
    body.addClass('no-images');
  }
  
  
  /* Event handlers */
  
  // Link handling
  document.addEventListener('click', function(e) {
    var ele = e.target;
    while (ele && ele.tagName && ele.tagName != 'A') ele = ele.parentNode;
    if (ele.tagName == 'A') {
      var link = ele;
      var typeOfLink = link.getAttribute('href').substring(0, 1);
      
      switch (typeOfLink) {
        // Wiki link
        case '/':
          link.onclick = function(e) {
            wikipedia.history.getCurrent().scrollTop = wdgtContent.scrollTop;
            body.addClass('loading');
            
            wikipedia.requestArticle(
              this.getAttribute('href').replace('/wiki/', ''),
              function(request) {
                if (request.status == 0) {
                  wdgtContent.innerHTML = errorMessage;
                  body.dropClass('loading');
                  return;
                }
                prepareArticle(request.responseText, false, function() {
                  if (wdgtMain.hasClass('content-active')) {
                    if (!wdgtMain.hasClass('notoc')) {
                      animatePanel(screen.availWidth, 0, function() {
                        wdgtMain.dropClass('content-active');
                      });
                    }
                    else {
                      animatePanel(0, -screen.availWidth);
                    }
                  }
                });
              }
            );
          };
          break;
        
        // Fragment identifier
        case '#':
          if (media != 'handheld' || wdgtMain.hasClass('content-active')) {
            link.addEventListener('click', function(e) {
              var id = this.getAttribute('href').substring(1);
              if (id) {
                try {
                  (document.getElementById(id) || document.getElementsByName(id)[0]).scrollIntoView();
                }
                catch(e) { }
              }
            }, false);
          }
          break;
        
        // More cases here?
        
        // Otherwise, it should be a "normal" link
        default:
          link.onclick = function(e) {
            widget.openURL(this.getAttribute('href'));
            e.preventDefault();
          };
      }
    }
  }, true);
  
  document.onkeydown = function(e) {
    // This is... not pretty.
    if (e.target.id != 'wdgt-query' && !body.hasClass('no-search')) {
      if (media == 'handheld') {
        if (!body.hasClass('config') && !wdgtMain.hasClass('notoc')) {
          // If the TOC is hidden
          if (wdgtMain.hasClass('content-active')) {
            if (e.keyCode == FORWARD_BUTTON) {
              wdgtForwardButton.onclick(null, function() {
                if (!wdgtMain.hasClass('notoc')) {
                  animatePanel(screen.availWidth, 0, function() {
                    wdgtMain.dropClass('content-active');
                  });
                }
                else {
                  animatePanel(0, -screen.availWidth);
                }
              });
            }
            else if (e.keyCode == BACK_BUTTON) {
              animatePanel(null, 0, function() {
                wdgtMain.dropClass('content-active');
              });
            }
          }
          // If the TOC is visible
          else {
            if (e.keyCode == FORWARD_BUTTON) {
              animatePanel(0, -screen.availWidth, function() {
                wdgtMain.addClass('content-active');
              });
            }
            else if (e.keyCode == BACK_BUTTON) {
              wdgtBackButton.onclick(null, function() {
                animatePanel(-screen.availWidth * 2, -screen.availWidth, function() {
                  wdgtMain.addClass('content-active');
                });
              });
            }
          }
        }
        else if (wdgtMain.hasClass('notoc')) {
          if (e.keyCode == FORWARD_BUTTON) {
            wdgtForwardButton.onclick(null, function() {
              if (!wdgtMain.hasClass('notoc')) {
                animatePanel(screen.availWidth, 0, function() {
                  wdgtMain.dropClass('content-active');
                });
              }
              else {
                animatePanel(0, -screen.availWidth);
              }
            });
          }
          else if (e.keyCode == BACK_BUTTON) {
            wdgtBackButton.onclick(null, function() {
              animatePanel(-screen.availWidth * 2, -screen.availWidth, function() {
                wdgtMain.addClass('content-active');
              });
            });
          }
        }
      }
      else {
        if (e.keyCode == BACK_BUTTON) wdgtBackButton.onclick();
        if (e.keyCode == FORWARD_BUTTON) wdgtForwardButton.onclick();
      }
    }
  };
  
  wdgtSearch.onsubmit = function(e) {
    clearTimeout(searchTimeout);
    
    if (wdgtQuery.value !== '') {
      var current = wikipedia.history.getCurrent();
      if (current) current.scrollTop = wdgtContent.scrollTop;
      body.dropClass('config');
      body.addClass('loading');
      
      if (minimized) minimizeRestore();
      
      var value = wdgtQuery.value;
      if (widget.preferenceForKey('lang') == 'ja') {
        value = normalizeRomaji(normalizeKana(value));
      }
      
      wikipedia.specialSearch(
        value.substring(0, 1).toUpperCase() + value.substring(1),
        function(request) {
          if (request.status == 0) {
            wdgtContent.innerHTML = errorMessage;
            body.dropClass('loading');
            return;
          }
          prepareArticle(request.responseText, false, function() {
            body.dropClass('no-search');
            if (wdgtMain.hasClass('content-active')) {
              animatePanel(screen.availWidth, 0, function() {
                wdgtMain.dropClass('content-active');
              });
            }
/*            if (wdgtMain.hasClass('notoc')) {
              animatePanel(0, -screen.availWidth, function() {
                wdgtMain.addClass('content-active');
              });		
            }	*/
          });
        }
      );
    }
  };
  
  wdgtQuery.onkeydown = function(e) {
    wdgtQueryList.innerHTML = '';
  };
  
  wdgtQuery.onkeyup = function(e) {
    if (autoSearch && autoSearchActive) {
      // "Special" keys shouldn't fire the search
      if (!(e.keyCode in KEYS.SPECIAL)) {
        // I have to blur(), then focus() to bring the datalist up again
        wdgtQuery.blur();
        wdgtQuery.focus();
        
        clearTimeout(searchTimeout);
        
        if (this.value !== '') {
          var value = this.value.substring(0, 1).toUpperCase() + this.value.substring(1);
          if (widget.preferenceForKey('lang') == 'ja') {
            value = normalizeRomaji(normalizeKana(value));
          }
          
          searchTimeout = setTimeout(function() {
            wdgtSearchLoader.addClass('loading');
            wikipedia.suggest(
              value,
              function(request) {
                try {
                  var responseBody = eval('(' + request.responseText + ')');
                  var links = responseBody.query.allpages;
                  for (var i = 0, link; link = links[i]; i++) {
                    wdgtQueryList.innerHTML += '<option value=' + link.title.replace(/\s+/g, '&#32;') + '>';
                  }
                  // I have to blur(), then focus() to bring the datalist up again
                  wdgtQuery.blur();
                  wdgtQuery.focus();
                  wdgtSearchLoader.dropClass('loading');
                }
                catch(e) { }
              }
            );
          }, 500);
        }
        else {
          wdgtSearchLoader.dropClass('loading');
        }
      }
//      if (e.keyCode == KEYS.ENTER) wdgtSearch.onsubmit();
    }
  };
  
  wdgtBackButton.onclick = function(e, callback) {
    if (wikipedia.history.back.length > 1) {
      var current = wikipedia.history.getCurrent();
      callback = callback || null;
      
      body.addClass('loading');
      current.scrollTop = wdgtContent.scrollTop;
      
      wikipedia.history.forward.unshift(wikipedia.history.back.shift());
      
      current = wikipedia.history.getCurrent();
      
      if (current.articleContent) {
        prepareArticle(current.articleContent, true, callback);
      }
      else {
        wikipedia.requestArticle(current.articleName, function(request) {
          prepareArticle(request.responseText, true, callback)
        });
      }
    }
  };
  
  wdgtForwardButton.onclick = function(e, callback) {
    if (wikipedia.history.forward.length) {
      var current = wikipedia.history.getCurrent();
      callback = callback || null;
      
      body.addClass('loading');
      current.scrollTop = wdgtContent.scrollTop;
      
      wikipedia.history.back.unshift(wikipedia.history.forward.shift());
      
      current = wikipedia.history.getCurrent();
      
      if (current.articleContent) {
        prepareArticle(current.articleContent, true, callback);
      }
      else {
        wikipedia.requestArticle(current.articleName, function(request) {
          prepareArticle(request.responseText, true, callback)
        });
      }
    }
  };
  
  wdgtTextupButton.onclick = function(e) {
    e.preventDefault();
    var currentFontSize = window.getComputedStyle(wdgtContent, null).getPropertyValue('font-size');
    var newFontSize = parseInt(currentFontSize) + 2 + 'px';
    wdgtContent.style.fontSize = newFontSize;
    widget.setPreferenceForKey(newFontSize, 'font-size');
  };
  
  wdgtTextdownButton.onclick = function(e) {
    e.preventDefault();
    var currentFontSize = window.getComputedStyle(wdgtContent, null).getPropertyValue('font-size');
    var newFontSize = parseInt(currentFontSize) - 2 + 'px';
    wdgtContent.style.fontSize = newFontSize;
    widget.setPreferenceForKey(newFontSize, 'font-size');
  };
  
  wdgtEditButton.onclick = function(e) {
    var baseUri = wikipedia.getBaseUri();
    widget.openURL(baseUri + 'w/index.php?action=edit&title=' + wikipedia.history.getCurrent().articleName);
  };
  
  wdgtCloseButton.onclick = function(e) {
    window.close();
  };
  
  wdgtConfigButton.onclick = function(e) {
    if (minimized) minimizeRestore();
    
    if (body.hasClass('config')) {
      if (media == 'handheld') {
        if (wdgtMain.hasClass('content-active')) {
          animatePanel(-screen.availWidth, 0, function() {
            wdgtMain.dropClass('content-active');
          });
        }
        else {
          animatePanel(-screen.availWidth * 2, -screen.availWidth, function() {
            wdgtMain.addClass('content-active');
          });
        }
      }
      
      body.dropClass('config');
    }
    else {
      body.addClass('config');
      
      if (media == 'handheld') {
        if (wdgtMain.hasClass('content-active')) {
          animatePanel(0, -screen.availWidth, function() {
            wdgtMain.dropClass('content-active');
          });
        }
        else {
          animatePanel(null, -screen.availWidth, function() {
            wdgtMain.addClass('content-active');
          });
        }
      }
    }
  };
  
  wdgtMinmaxButton.onclick = function(e) {
    minimizeRestore();
  };
  
  wdgtLangPicker.onchange = function(e) {
    var lang = this.value;
    document.toLocale(lang);
    wikipedia.setLanguage(lang);
    widget.setPreferenceForKey(lang, 'lang');
  };
  
  wdgtLoadMain.onchange = function(e) {
    widget.setPreferenceForKey(this.checked, 'load-main');
  };
  
  wdgtUseAutoSearch.onchange = function(e) {
    autoSearchActive = this.checked;
    // Delete the items in the auto search list
    wdgtQueryList.innerHTML = '';
    widget.setPreferenceForKey(this.checked, 'use-auto-search')
  };
  
  wdgtLoadImages.onchange = function(e) {
    this.checked ? body.dropClass('no-images') : body.addClass('no-images');
    widget.setPreferenceForKey(this.checked, 'load-images')
  };
  
  wdgtTextupButton.onmouseover = function(e) {
    wdgtStatus.innerText = wdgtTextupButton.innerText;
  };
  
  wdgtTextdownButton.onmouseover = function(e) {
    wdgtStatus.innerText = wdgtTextdownButton.innerText;
  };
  
  wdgtEditButton.onmouseover = function(e) {
    wdgtStatus.innerText = wdgtEditButton.innerText;
  };
  
  wdgtTextupButton.onmouseout =
  wdgtTextdownButton.onmouseout =
  wdgtEditButton.onmouseout = function(e) {
    wdgtStatus.innerText = '';
  };
  
  wdgtSwitchContent.onclick = function(e) {
    // hacky as *beep*. But only temp
    document.onkeydown({target: {id: null}, keyCode: FORWARD_BUTTON});
  }
  
 wdgtSwitchToc.onclick = function(e) {
    // hacky as *beep*. But only temp
    document.onkeydown({target: {id: null}, keyCode: BACK_BUTTON});
  }
  
  
  /* Functions */
  
  function prepareArticle(articleContent, historyBrowsing, callback) {
    var current;
    var articleName = null;
    var articleTitle = null;
    var otherLang = null;
    
    // This is hacky, but currently needed
    temp.innerHTML = articleContent;
    
    // Grab the article name
    try {
      articleTitle = document.getElementById('content').getElementsByTagName('h1')[0].firstChild.nodeValue;
      articleName = document.getElementById('ca-nstab-main').getElementsByTagName('a')[0].getAttribute('href').replace('/wiki/', '');
    }
    catch(e) { }
    
    // Display the content
    var header = document.getElementsByTagName('h1')[1];
    var content = document.getElementById('content');
    try {
      wdgtContent.innerHTML = content.innerHTML;
    }
    catch(e) {
      wdgtContent.innerHTML = errorMessage;
      body.dropClass('loading');
      return;
    }
    
    // Wikipedia content IDs
    var top = document.getElementById('top');
    var toc = document.getElementById('toc');
    
    // Add to history
    handleHistory(articleName, articleTitle, articleContent, historyBrowsing);
    
    // Scroll to the top, or where we were
    current = wikipedia.history.getCurrent();
    wdgtContent.scrollTop = current ? current.scrollTop : 0;
    
    // Grab the table of contents and display it on its own
    if (toc && toc.getElementsByTagName('ul')[0]) {
      wdgtMain.dropClass('notoc');
      
      if (media == 'handheld') wdgtToc.innerHTML = '<h2>' + articleTitle + '</h2>';
      
      wdgtToc.innerHTML = '';
      wdgtToc.innerHTML += '<ul><li><a href="#top" id="top-link">^</a></li>' +
        toc.getElementsByTagName('ul')[0].innerHTML + '</ul>';
    }
    else {
      wdgtMain.addClass('notoc');
      wdgtToc.innerHTML = '';
    }
    
    // On mobile, assign special actions to TOC links, to hide TOC on click
    if (media == 'handheld') {
      for (var i = 0, link; link = wdgtToc.getElementsByTagName('A')[i]; i++) {
        link.addEventListener('click', function() {
          animatePanel(0, -screen.availWidth, function(link) {
            return function() {
              var id = link.getAttribute('href').substring(1);
              if (id) {
                try {
                  (document.getElementById(id) || document.getElementsByName(id)[0]).scrollIntoView();
                }
                catch(e) { }
              }
            }
          }(this));
          wdgtMain.addClass('content-active');
        }, false);
      }
    }
    
    if (callback) callback();
    
    // We're done. Stop the loading inidcator
    body.dropClass('loading');
    
    // Now we can enable some buttons (only necessary the first time)
    wdgtTextupButton.disabled = false;
    wdgtTextdownButton.disabled = false;
    wdgtEditButton.disabled = false;
  }
  
  
  function handleHistory(articleName, articleTitle, articleContent, historyBrowsing) {
    if (wikipedia.history.back[0]) {
      // Don't add another history entry if it's the same article
      if (wikipedia.history.back[0].articleTitle != articleTitle) {
        addToHistory(articleName, articleContent);
        
        // Delete forward history on a new "browsing branch"
        if (!historyBrowsing) wikipedia.history.forward.length = 0;
      }
    }
    else {
      addToHistory(articleName, articleContent);
    }
    
    // Add to history
    function addToHistory(articleName, articleContent) {
      wikipedia.history.back.unshift({
        articleName: (articleName && articleName.substring(0, 1) != '/' ) ? articleName : articleTitle,
        articleTitle: articleTitle,
        articleContent: contentCache ? articleContent : null
      });
    }
    
    // Delete items when history is filled up
    if (wikipedia.history.back.length >= historyLength + 1) {
      wikipedia.history.back.length = historyLength + 1;
    }
    
    // Handle back and forward buttons disabling
    if (wikipedia.history.back[1]) {
      wdgtBackButton.removeAttribute('disabled');
    }
    else {
      wdgtBackButton.setAttribute('disabled', 'disabled');
      wdgtBackButton.blur();
    }
    
    if (wikipedia.history.forward[0] && wikipedia.history.forward[0].articleName) {
      wdgtForwardButton.removeAttribute('disabled');
    }
    else {
      wdgtForwardButton.setAttribute('disabled', 'disabled');
      wdgtForwardButton.blur();
    }
  }
  
  
  
  function animatePanel(from, to, callback) {
    body.addClass('animating');
    clearInterval(animInterval);
    from = from != null ? from : parseInt(wdgtMain.style.left, 10);
    wdgtMain.style.left = from + 'px';
    
    if (from > to) {
      var animInterval = setInterval(function() {
        if ((from - animationSteps) > to) {
          wdgtMain.style.left = from - animationSteps + 'px';
          from = from - animationSteps;
        }
        else {
          clearInterval(animInterval);
          wdgtMain.style.left = to + 'px';
          setTimeout(function() {
            if (callback) callback();
            body.dropClass('animating');
          }, 0);
        }
      }, 100);
    }
    else if (from < to) {
      var animInterval = setInterval(function() {
        if ((from + animationSteps) < to) {
          wdgtMain.style.left = from + animationSteps + 'px';
          from = from + animationSteps;
        }
        else {
          clearInterval(animInterval);
          wdgtMain.style.left = to + 'px';
          setTimeout(function() {
            if (callback) callback();
            body.dropClass('animating');
          }, 0);
        }
      }, 100);
    }
  }
  
  
  
  function minimizeRestore() {
    if (minimized) {
      if (media == 'screen') window.resizeTo(widget.preferenceForKey('width'), widget.preferenceForKey('height'));
      body.dropClass('minimized');
    }
    else {
      if (media == 'screen') window.resizeTo(400, 70);
      body.addClass('minimized');
    }
    minimized = !minimized;
  }
  
  
  
  function set() {
    var result = {};
    for (var i = 0, length = arguments.length; i < length; i++) {
      result[arguments[i]] = 1;
    }
    return result;
  }
};
